@@ -23,7 +23,7 @@
 #include <linux/lkst_events.h>
 
 #define NAME	"lkstlogd"
-#define VERSION "1.1"
+#define VERSION "1.2"
 
 #define LOG_HEADER_SIZE \
 	(sizeof(log_header_t) +					\
@@ -62,6 +62,7 @@ void restore_buffer(void);
 void write_header(FILE*);
 
 int read_on = 0;
+int term_flg = 0;
 int restart = 0;
 int file_snum = 0;
 int change_fd = 0;
@@ -143,8 +144,6 @@ int main(int argc, char** argv)
 
 	owner_daemon();
 
-	cleanup();
-
 	exit(0);
 }
 
@@ -244,6 +243,7 @@ int owner_daemon()
 				cpu_id = i;
 				info[i].buf_size = bufsize[i];
 				child_daemon_main();
+				exit(0);
 			}
 		}
 
@@ -253,6 +253,7 @@ int owner_daemon()
 			waitpid((pid_t)info[i].c_pid, &status, 0);
 		}
 
+		cleanup();
 		if(restart == 0)
 			break;
 
@@ -291,12 +292,14 @@ int child_daemon_main()
 	if(signal(SIGINT, SIG_DFL) == SIG_ERR) exit(1);
 	if(signal(SIGTERM, child_die_handler) == SIG_ERR) exit(1);
 
-	if(read_on == 1)
-		child_buf_read();
+	term_flg = 0;
 
-	/* Waiting for SIGUSER1 */
-	while (1)
-		pause();
+	while (!term_flg){
+		if (read_on)
+		       	child_buf_read();
+		else 
+			pause();
+	}
 
 	return 0;
 }
@@ -522,6 +525,7 @@ void init_buffer(int devfd)
 			if (lp.listent[k].id == info[i].buf_id[0]) {
 				info[i].prev_id = lp.listent[k].prev;
 				info[i].prev_size = lp.listent[lp.listent[k].prev].size;
+				break;
 			}
 		}
 	}
@@ -537,21 +541,23 @@ void restore_buffer(void)
 	devfd = open_dev();
 	
 	for (i=0; i<ncpu; i++) {
-		ret = ioctl(devfd, LKST_IOC_BUFFER_DELETE, info[i].prev_id);
-		if(!ret){
+		do {
+			ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
+			ioctl(devfd, LKST_IOC_BUFFER_DELETE, info[i].prev_id);
 			param.id = info[i].prev_id;
 			param.size = info[i].prev_size;
 			param.cpu = i;
 			ret = ioctl(devfd, LKST_IOC_BUFFER_CREATE, &param);
-		}
+		} while(ret);
 
 		for (j=0; j<init_buf_num; j++){
 		retry:
 			ret = ioctl(devfd, LKST_IOC_BUFFER_DELETE, info[i].buf_id[j]);
-			if (ret == EBUSY) {
-				ret = ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
+			if ( ret == 0 ) continue;
+			if (errno == EBUSY) {
+				ioctl(devfd, LKST_IOC_BUFFER_SHIFT, i);
 				goto retry;
-			} else if (ret) {
+			} else {
 				fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_DELETE) error: %s\n", strerror(errno));
 				exit(1);
 			}
@@ -657,9 +663,6 @@ int buffer_read_size(int buf_size)
 
 void cleanup()
 {
-	read_on = 0;
-
-	kill_children(SIGTERM);
 
 	restore_maskset();
 	restore_buffer();
@@ -669,22 +672,20 @@ void cleanup()
 
 void sighup_handler()
 {
-	cleanup();
+	kill_children(SIGTERM);
 	restart = 1;
-	
-	return;
 }
 
 void die_handler(int sigid)
 {
-	cleanup();
-	exit(0);
+	kill_children(SIGTERM);
+	restart = 0;
 }
 
 void child_die_handler(int sigid)
 {
 	read_on = 0;
-	exit(0);
+	term_flg = 1;
 }
 
 void par_snap_handler()
@@ -758,8 +759,6 @@ void child_snap_handler()
 
 	read_on = 1;
 
-	child_buf_read();
-
 	return;
 }
 
@@ -771,7 +770,8 @@ void buffer_setrmod(int devfd)
 	sp.cpu = cpu_id;
 	sp.mode = STD;
 
-	if (ret = ioctl(devfd, LKST_IOC_BUFFER_SETRMOD, &sp)) {
+	ret = ioctl(devfd, LKST_IOC_BUFFER_SETRMOD, &sp);
+	if( ret ) {
 		fprintf(stderr, "lkstlogd error: ioctl(LKST_IOC_BUFFER_SETRMOD) error: %s\n", strerror(errno));
 		exit(1);
 	}
